
;;; org-zotero-export --- export Zotero cites to ODT
;;
;; Copyright (C) 2013 Christian Moe
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;
;;; Comments:
;; 
;; Adds a zotero link type. The path is the Zotero item ID. The
;; description is parsed to determine how Zotero should format the
;; citation. The description should be in the format:
;; 
;;   (prenote: author, date: p.number: postnote) 
;;
;; e.g.
;;
;;   (Jones et al., 2000)
;;   (see: Jones et al., 2000: p.24:, see also the update)
;;   (2000: p.47)
;;
;; Only the date is required, all other parts are optional. The date
;; must be a four-digit year or "n.d.". Page numbers, if supplied,
;; should be prefixed with "p." to distinguish them from other
;; post-notes.
;; 
;; Multiple citations are allowed:
;;
;;   [[zotero:0_T3U33WR8;0_WMNA92GP][(McCright, 2011: p.5; Washington, 2011: p.26)]].
;;
;; As an experimental hack, org-zotero can fake textual cites though
;; Zotero does not support them. See the docstring of
;; org-zotero-use-textual-cites.

(org-add-link-type "zotero" 
                   'org-zotero-open
                   'org-zotero-export)

(defvar org-zotero-loc-re
  "\\(@\\|pp?\\.\\)")

(defcustom org-zotero-use-textual-cites nil
  "If t, links without parentheses around the link description
will be handled as `textual' cites (with the default nil, they
will be handled as `custom' cites, which are retained verbatim
and are not updated by Zotero). Zotero does not provide an
equivalent of natbib's \citet command, so in order to fake
textual cites, one needs an appropriately customized CSL style
that does not bracket citations. When this variable is set to t,
therefore, parenthesized links will be exported with brackets
placed around the field code. Note that these brackets are not
part of the field code, and will not be updated in Zotero."
  ;; TODO add customization args
)

(defun org-zotero-open (path)
  (browse-url (format "zotero://select//%s" path)))

(defun org-zotero-export (path desc format)
  (cond 
   ((eq format 'odt)
    (let 
        ((refmark "<text:reference-mark-start text:name=\"%s\"/>%s<text:reference-mark-end text:name=\"%s\"/>") 
         (citation-id (substring (org-id-uuid) -10))
         (rnd (concat "RND" (substring (org-id-uuid) -10))) ;; FIXME check uniqueness?
         paths library-id key-item
         descs citeparts zitem custom 
         authdate citeyear prefix loc loctype suffix
         parenthesized c)
      ;; If desc is not parenthesized, this is
      ;; either a textual citation (if we're set to fake them)
      ;; or a custom citation to be passed verbatim
      (setq parenthesized 
            (and (string= "(" (substring desc 0 1))
                 (string= ")" (substring desc -1))))

      (if parenthesized
          (setq desc (substring desc 1 -1))
        (unless org-zotero-use-textual-cites
            (setq custom desc)))

      (when (and org-zotero-use-textual-cites
                 parenthesized)
        (setq refmark (format "(%s)" refmark)))
      
      ;; Start putting together the fieldcode
      (setq zitem
            (concat
             "ZOTERO_ITEM {"
             "\"citationID\":\"" citation-id "\","
             (when custom
               (concat
                "\"properties\": {"
                (format "\"custom\": \"%s\"" custom)
                "},"))
             "\"citationItems\":["))

      ;; If this is not a custom cite, parse it
      (unless custom

        ;; If multiple works parse all of them
        (setq paths (split-string path ";\\s-*"))
        (setq descs (split-string desc ";\\s-*"))
        (unless (= (length paths) (length descs))
          (error "Mismatched path/desc length in Zotero cite [%s][%s]" path desc))
        (setq c 1) ; work counter
        (while descs

          ;; Parse the description
          (setq library-id (car (split-string (car paths) "_"))
                item-key (car (cdr (split-string (car paths) "_"))))
          (setq citeparts (split-string (car descs) "\\s-*:\\s-*"))
          ;; If there's a date in the first section, it's author-date
          (if (string-match "\\([0-9]\\{4\\}\\|n\\.d\\.\\)" (car citeparts))
              (setq authdate (car citeparts)
                    prefix nil) ; 
            ;; Else first is prefix, second is authordate. 
            (setq prefix (car citeparts)
                  citeparts (cdr citeparts)
                  authdate (car citeparts)))
          ;; If the author-date part has only date
          (if (= (length authdate) 4) ; FIXME Improve on this!
              (setq citeyear t))
          ;; Figure out what of the rest is locator and what's suffix, if any
          (setq citeparts (cdr citeparts))
          (when citeparts
            ;; If there are two, it's easy
            (if (= 2 (length citeparts))
                (setq loc (car citeparts)
                      suffix (car (cdr citeparts)))
              ;; Else check for a locator label
              (if (string-match org-zotero-loc-re (car citeparts))
                  (setq loc (car citeparts))
                (setq suffix (car citeparts))))
            ;; Clean up the locator and set the label
            (when loc
              (string-match (concat org-zotero-loc-re " ?\\(.*\\)") loc)
              (let ((s (match-string 1 loc))) 
                (setq label
                      (cond 
                       ((string= s "@") "page")
                       ((string= s "p.") "page"))))
              (setq loc (match-string 2 loc))))

          ;; Complete the Zotero item
          (setq zitem 
                (concat
                 zitem
                 "{"
                 (when citeyear
                   "\"suppress-author\": true,")
                 (when prefix
                   (format "\"prefix\": \"%s\"," prefix))
                 (when loc
                   (format "\"locator\": \"%s\"," loc))
                 (when loctype
                   (format "\"label\": \"%s\"," loctype))
                 (when suffix
                   (format "\"suffix\": \"%s\"," suffix))
                 "\"uri\":["
                 "\"http://zotero.org/users/local/" library-id "/items/" item-key "\""
                 "]"
                 "}"
                 (when (cdr descs) ","))); if there are more works to follow

          ;; Loop to next work
          (setq c (+ 1 c))
          (setq paths (cdr paths))
          (setq descs (cdr descs))))

      ;; end fieldcode
      (setq zitem (concat zitem "]} " rnd))
      (setq zitem (replace-regexp-in-string "\"" "&quot;" zitem))
      (format refmark zitem desc zitem)))
   (t desc)))
